@@ -1,5 +1,5 @@
 /* grecs - Gray's Extensible Configuration System
-   Copyright (C) 2007-2016 Sergey Poznyakoff
+   Copyright (C) 2007-2025 Sergey Poznyakoff
 
    Grecs is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -267,20 +267,10 @@ grecs_tree_free(struct grecs_node *node)
 
 
 static int
-fake_callback(
-#if GRECS_TREE_API
-	enum grecs_callback_command cmd,
-	grecs_node_t *node,
-	void *varptr,
-	void *cb_data
-#else	
-	enum grecs_callback_command cmd,
-	grecs_locus_t *locus,
-	void *varptr,
-	grecs_value_t *value,
-	void *cb_data
-#endif	
-	)
+fake_callback(enum grecs_callback_command cmd,
+	      grecs_node_t *node,
+	      void *varptr,
+	      void *cb_data)
 {
 	return 0;
 }
@@ -737,40 +727,28 @@ struct grecs_prop grecs_prop_tab[] = {
  (sizeof(grecs_prop_tab) / sizeof(grecs_prop_tab[0]))
 
 void
-grecs_process_ident(struct grecs_keyword *kwp, grecs_value_t *value,
-		     void *base, grecs_locus_t *locus)
+grecs_process_ident(struct grecs_keyword *kwp, grecs_node_t *node, void *base)
 {
 	void *target;
-
+	grecs_value_t *value = node->v.value;
+		
 	if (!kwp)
 		return;
 	
 	target = target_ptr(kwp, (char *) base);
 	
 	if (kwp->callback) {
-#if GRECS_TREE_API
-		struct grecs_node node = { 0 };
-		node.locus = *locus;
-		node.v.value = value;
-		node.ident = (char*) kwp->ident;
 		kwp->callback(grecs_callback_set_value,
-			      &node,
+			      node,
 			      target,
 			      &kwp->callback_data);
-#else
-		kwp->callback(grecs_callback_set_value,
-			      locus,
-			      target,
-			      value,
-			      &kwp->callback_data);
-#endif
 	} else if (kwp->type == grecs_type_void || target == NULL)
 		return;
 	else if (!value) {
-		grecs_error(locus, 0, "%s has no value", kwp->ident);
+		grecs_error(&node->locus, 0, "%s has no value", kwp->ident);
 		return;
 	} else if (value->type == GRECS_TYPE_ARRAY) {
-		grecs_error(locus, 0,
+		grecs_error(&node->locus, 0,
 			     _("too many arguments to `%s'; missing semicolon?"),
 			     kwp->ident);
 		return;
@@ -784,7 +762,7 @@ grecs_process_ident(struct grecs_keyword *kwp, grecs_value_t *value,
 			
 			if (type >= grecs_prop_count
 			    || (size = grecs_prop_tab[type].size) == 0) {
-				grecs_error(locus, 0,
+				grecs_error(&node->locus, 0,
 					     _("INTERNAL ERROR at %s:%d: "
 					       "unhandled data type %d"),
 					     __FILE__, __LINE__, type);
@@ -819,7 +797,7 @@ grecs_process_ident(struct grecs_keyword *kwp, grecs_value_t *value,
 			}
 			*(struct grecs_list**)target = list;
 		} else {
-			grecs_error(locus, 0,
+			grecs_error(&node->locus, 0,
 				     _("incompatible data type for `%s'"),
 				     kwp->ident);
 			return;
@@ -832,7 +810,7 @@ grecs_process_ident(struct grecs_keyword *kwp, grecs_value_t *value,
       
 		if (type >= grecs_prop_count
 		    || (size = grecs_prop_tab[type].size) == 0) {
-			grecs_error(locus, 0,
+			grecs_error(&node->locus, 0,
 				     _("INTERNAL ERROR at %s:%d: unhandled data type %d"),
 				     __FILE__, __LINE__, type);
 			abort();
@@ -886,18 +864,10 @@ stmt_begin(struct nodeproc_closure *clos,
 		target = target_ptr(kwp, CURRENT_BASE(clos));
 		clos->cursect = kwp;
 		if (kwp->callback) {
-#if GRECS_TREE_API			
 			if (kwp->callback(grecs_callback_section_begin,
 					  node,
 					  target,
 					  &kwp->callback_data))
-#else
-			if (kwp->callback(grecs_callback_section_begin,
-					  &node->locus,
-					  target,
-					  node->v.value,
-					  &kwp->callback_data))
-#endif
 				clos->cursect = &fake;
 		} else
 			kwp->callback_data = target;
@@ -922,18 +892,10 @@ stmt_end(struct nodeproc_closure *clos, struct grecs_node *node)
 	if (!clos->cursect)
 		abort();
 	if (callback)
-#if GRECS_TREE_API			
 		callback(grecs_callback_section_end,
 			 node,
 			 kwp ? target_ptr(kwp, CURRENT_BASE(clos)) : NULL,
 			 dataptr);
-#else		
-		callback(grecs_callback_section_end,
-			 &node->locus,
-			 kwp ? target_ptr(kwp, CURRENT_BASE(clos)) : NULL,
-			 NULL,
-			 dataptr);
-#endif
 	if (kwp)
 		kwp->callback_data = NULL;
 }
@@ -949,8 +911,7 @@ nodeproc(enum grecs_tree_recurse_op op, struct grecs_node *node, void *data)
 		kwp = find_keyword(clos->cursect, node);
 		if (!kwp)
 			return grecs_tree_recurse_skip;
-		grecs_process_ident(kwp, node->v.value, CURRENT_BASE(clos),
-				    &node->idloc);
+		grecs_process_ident(kwp, node, CURRENT_BASE(clos));
 		break;
 		
 	case grecs_tree_recurse_pre:
