@@ -1,5 +1,5 @@
 /* This file is part of GNU Pies.
-   Copyright (C) 2015-2022 Sergey Poznyakoff
+   Copyright (C) 2015-2026 Sergey Poznyakoff
 
    GNU Pies is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
 
 #include <config.h>
 #include <stdlib.h>
+#include <string.h>
 #include <grecs.h>
 #include <sysexits.h>
 #include <wordsplit.h>
@@ -40,7 +41,6 @@ pp_add_option (const char *opt, const char *arg)
 	  exit (EX_CONFIG);
 	}
       pp_cmd = grecs_txtacc_create ();
-      grecs_txtacc_grow_string (pp_cmd, grecs_preprocessor);
     }
   grecs_txtacc_grow_char (pp_cmd, ' ');
   grecs_txtacc_grow_string (pp_cmd, opt);
@@ -59,19 +59,25 @@ pp_add_option (const char *opt, const char *arg)
     }
 }
 
+/*
+ * Build and return the preprocessor command line.
+ */
 char *
 pp_command_line (void)
 {
   char *ret;
+  size_t len, pplen;
   
   if (!pp_cmd)
     {
       if (!grecs_preprocessor)
 	return NULL;
+
+      /* Initialize pp_cmd */
       pp_cmd = grecs_txtacc_create ();
-      grecs_txtacc_grow_string (pp_cmd, grecs_preprocessor);
     }
 
+  /* Append the default search path options. */
   if (pp_path)
     {
       struct wordsplit ws;
@@ -89,9 +95,34 @@ pp_command_line (void)
 	}
       wordsplit_free (&ws);
     }
+
+  /*
+   * Insert preprocessor command before its options.
+   * Since grecs_txtacc interface makes no provision for inserting text
+   * at the beginning, this is done as follows:
+   *
+   *  1. Terminating \0 is appended.
+   *  2. The preprocessor command line is appended *to the end* of the
+   *     buffer to make sure sufficient space is allocated.
+   *  3. The txtacc is finalized.  This returns allocated array of
+   *     characters with the preprocessor name appended after the \0
+   *     character.
+   *  4. The resulting array is shifted N bytes to the right, where
+   *     N is the length of the preprocessor command plus one.  Now
+   *     terminating \0 occupies the last byte of the allocated memory.
+   *  5. Preprocessor command is placed at the beginning of the array,
+   *     filling in the space provided in previous step.
+   */
   
   grecs_txtacc_grow_char (pp_cmd, 0);
+  grecs_txtacc_grow_string (pp_cmd, grecs_preprocessor);
   ret = grecs_txtacc_finish (pp_cmd, 1);
+
+  len = strlen (ret);
+  pplen = strlen (grecs_preprocessor);
+  memmove (ret + pplen, ret, len + 1);
+  memcpy (ret, grecs_preprocessor, pplen);
+  
   grecs_txtacc_free (pp_cmd);
   pp_cmd = NULL;
   return ret;
